package com.supwisdom.dlpay.api.controller

import com.supwisdom.dlpay.api.bean.OpenShopParam
import com.supwisdom.dlpay.api.bean.QueryShopParam
import com.supwisdom.dlpay.framework.ResponseBodyBuilder
import com.supwisdom.dlpay.api.service.ShopService
import com.supwisdom.dlpay.exception.RequestParamCheckException
import com.supwisdom.dlpay.exception.TransactionException
import com.supwisdom.dlpay.framework.service.SystemUtilService
import com.supwisdom.dlpay.framework.util.TradeErrorCode
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.net.URLDecoder

@RestController
@RequestMapping("/api/shop")
class ShopAPIController {
    @Autowired
    private lateinit var shopService: ShopService
    @Autowired
    private lateinit var systemUtilService: SystemUtilService

    @PostMapping("/open")
    fun openAccount(@RequestBody request: OpenShopParam): ResponseEntity<Any> {
        return try {
            if (request.checkParam() && request.checkSign(systemUtilService.getSubsystemSignKey(request.syscode))) {
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(TradeErrorCode.REQUEST_SIGN_ERROR, "参数签名错误"))
            }
            shopService.findByThirdUniqueId(request.shopUniqueId)?.let {
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(TradeErrorCode.BUSINESS_SHOP_EXISTS, "该商户唯一号[${request.shopUniqueId}]已经存在"))
            }

            request.shopname = request.shopname?.let { URLDecoder.decode(request.shopname, Charsets.UTF_8.toString()) } //解码
            request.contactman = request.contactman?.let { URLDecoder.decode(request.contactman, Charsets.UTF_8.toString()) } //解码
            request.address = request.address?.let { URLDecoder.decode(request.address, Charsets.UTF_8.toString()) } //解码
            shopService.registerShop(request)?.let {
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .data("shopid", it.shopid)
                        .data("shopaccno",it.shopaccno?:"")
                        .success("注册成功"))
            }

            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .fail(TradeErrorCode.BUSINESS_DEAL_ERROR, "商户注册失败"))
        } catch (ex: RequestParamCheckException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .requestException(ex, "请求参数错误"))
        } catch (et: TransactionException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .transException(et, "业务处理错误"))
        }
    }

    @GetMapping("/query")
    fun queryShop(@RequestBody request: QueryShopParam): ResponseEntity<Any> {
        return try {
            if (request.checkParam() && request.checkSign(systemUtilService.getSubsystemSignKey(request.syscode))) {
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(TradeErrorCode.REQUEST_SIGN_ERROR, "参数签名错误"))
            }

            shopService.findByShopBySearch(request.shopid,request.shopaccno,request.shopUniqueId)?.let {
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .data("shopid", it)
                        .success())
            }

            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .fail(TradeErrorCode.SHOP_NOT_EXISTS, "商户不存在"))
        } catch (ex: RequestParamCheckException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .requestException(ex, "请求参数错误"))
        } catch (et: TransactionException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .transException(et, "业务处理错误"))
        }
    }
}